#include "TextureLoader.h"
#include <map>

class PictureInformation
{
	int frameId;
	Entity* entity;

public:
	PictureInformation(){}
	PictureInformation(Entity* entity, int imageId)
		:entity(entity),frameId(-1)
	{
		char fileName[100];
		sprintf(fileName,"images/img%i.jpg",imageId);
		OpenSGModel* imageModel = dynamic_cast<OpenSGModel*>(entity->getVisualRepresentation());
		double aspect = SetTexture(imageModel->getSubNodeByName("Image"),fileName);
		NodePtr nodePtr = imageModel->getNodePtr();
		OpenSGTransformationSceneGraphNode transformNode(nodePtr);
		TransformationData transform = transformNode.getTransformation();
		if(aspect>1.0) transform.scale.set(aspect,1.0,1.0);
		else transform.scale.set(1.0,1.0/aspect,1.0);
		transformNode.setTransformation(transform);

		nextFrame();
	}

	void nextFrame()
	{
		char fileName[100];
		bool notFound = true;
		frameId++;
		while(notFound)
		{
			sprintf(fileName,"frames/frame%i.jpg",frameId);
			FILE* file = fopen(fileName,"rb");
			if(file)
			{
				notFound = false;
				fclose(file);
			}
			else
			{
				if(frameId==0) return;
				frameId = 0;
			}
		}
		ModelInterface* imageModel = entity->getVisualRepresentation();
		SetTexture(imageModel->getSubNodeByName("Frame"),fileName);
	}
};

typedef std::map<unsigned int,PictureInformation> PictureMap;


class PictureLoader: public AbstractEntityCreationCB
{
	int imageId;
	PictureMap* pictureMap;

public:
	PictureLoader(int imageId, PictureMap* pictureMap)
		:imageId(imageId),pictureMap(pictureMap)
	{
	}

	void call(Entity* newEntity)
	{
		(*pictureMap)[newEntity->getEnvironmentBasedId()] = PictureInformation(newEntity,imageId);
	}
};

class PictureCave: public OpenSGApplicationBase, public PictureServer
{
	NetworkInterface* network;
	int imageId;
	Entity* selectedEntity;
	PictureMap pictureMap;
	AvatarInterface* avatar;
	ControllerInterface* controller;
	bool scale;
public:
	PictureCave() 
		:imageId(0),
		scale(false),
		selectedEntity(NULL)
	{
		
	}

	~PictureCave() 
	{
		globalCleanup();
	} 

	std::string getConfigFile(const CommandLineArgumentWrapper& args) 
	{
		if (args.containOption("config")) return args.getOptionValue("config");
		else return "config/general.xml";
	}

	bool initialize(const CommandLineArgumentWrapper& args) 
	{
		OpenSGSceneGraphInterface* sgIF = dynamic_cast<OpenSGSceneGraphInterface*>(sceneGraphInterface);
		// must exist because it is created by the OutputInterface
		if (!sgIF) {
			printd(ERROR, "PictureCAVE::initialize(): Unable to obtain SceneGraphInterface!\n");
			return false;
		} // if

		// obtain the scene node from the SceneGraphInterface
		NodePtr scene = sgIF->getNodePtr();
		PointLightPtr pLight = PointLight::create();
		
		beginEditCP(pLight);
			//Attenuation parameters
			pLight->setConstantAttenuation (0.0001);
			pLight->setLinearAttenuation   (0);
			pLight->setQuadraticAttenuation(0.001);

			//color information
			pLight->setDiffuse (Color4f(1,   1,   1,   1));
			pLight->setAmbient (Color4f(0.2, 0.2, 0.2, 1));
			pLight->setSpecular(Color4f(1,   1,   1,   1));
		endEditCP(pLight);

		beginEditCP(scene);
			scene->setCore(pLight);
		endEditCP(scene);		

		// set root node to the responsible SceneManager (managed by OpenSGApplicationBase)
		setRootNode(scene);

		// set our transformation to the start transformation
		TransformationData startTrans =
			WorldDatabase::getEnvironmentWithId(1)->getStartTransformation(0);
		localUser->setNavigatedTransformation(startTrans);

		localUser = UserDatabase::getLocalUser();
		if (!localUser) {
			printd(ERROR, "Error: Could not find localUser!\n");
			return -1;
		}

		avatar = localUser->getAvatar();
		if (!avatar) {
			printd(ERROR, "Error: Could not find avatar!\n");
			return -1;
		}
		avatar->showAvatar(false);
		controller = controllerManager->getController();
		controller->registerButtonChangeCallback(new ControllerButtonChangeCB<PictureCave>(controller,this,&PictureCave::buttonChanged));
		if (!controller) {
			printd(ERROR,
					"PictureCave::initialize(): unable to obtain controller! Check ControllerManager configuration!\n");
			return false;
		}
		setHeadlight(false);
		accept(8085);
		return true;
	}

	void messageReceived(NetMessage* message)
	{
		char fileName[100];
		sprintf(fileName,"images/img%i.jpg",imageId);
		FILE* file = fopen(fileName,"wb");
		size_t size = message->getUInt32();
		while(size>0)
		{
			uint8_t c = message->getUInt8();
			fwrite(&c,1,1,file);
			size--;
		}
		fclose (file);
		TransformationData transform = identityTransformation();
		transform.position.set(0.0f,0.0f,1.0f);
		WorldDatabase::createEntity(0,1,transform,new PictureLoader(imageId,&pictureMap));
		imageId++;
	}

	void display(float dt) 
	{
		selectedEntity = localUser->getAssociatedEntityByIndex(0);
		if(selectedEntity && pictureMap.count(selectedEntity->getEnvironmentBasedId())==0) selectedEntity = NULL;

		if(!selectedEntity)
		{
			if(scale)
			{
				scale = false;
				EventManager::sendEvent(new NavigationResumeEvent(NAVIGATION_MODULE_ID),EventManager::EXECUTE_LOCAL);
			}
			return;
		}
		if(scale)
		{
			OpenSGModel* imageModel = dynamic_cast<OpenSGModel*>(selectedEntity->getVisualRepresentation());
			NodePtr nodePtr = imageModel->getNodePtr();
			OpenSGTransformationSceneGraphNode transformNode(nodePtr);
			TransformationData transform = transformNode.getTransformation();
			float scaleValue = controller->getAxisValue(1)*dt;
			if(transform.scale[0]+scaleValue>0.2 && transform.scale[1]+scaleValue>0.2)
			{
				transform.scale[0]+=scaleValue;
				transform.scale[1]+=scaleValue;
				transformNode.setTransformation(transform);
			}
		}
	} 

	void buttonChanged(int buttonIndex, int newButtonValue)
	{
		if(selectedEntity)
		{
			if(buttonIndex==1 && newButtonValue==1)
			{
				pictureMap[selectedEntity->getEnvironmentBasedId()].nextFrame();
			}
			if(buttonIndex==2 && newButtonValue==1)
			{
				if(scale)
				{
					scale = false;
					EventManager::sendEvent(new NavigationResumeEvent(NAVIGATION_MODULE_ID),EventManager::EXECUTE_LOCAL);
				}
				else
				{
					EventManager::sendEvent(new NavigationPauseEvent(NAVIGATION_MODULE_ID),EventManager::EXECUTE_LOCAL);
					scale = true;
				}
			}
		}
	}

	void cleanup() 
	{
		close();
	}

	void initCoreComponentCallback(CoreComponents comp) 
	{
		if (comp == TRANSFORMATIONMANAGER) 
		{
			TransformationManager::registerModifierFactory(new HeightMapModifierFactory());
			TransformationManager::registerModifierFactory(new CheckCollisionModifierFactory());
		}
	}

	void initModuleCallback(ModuleInterface* module) 
	{
		if (module->getName() == "Network") 
		{
			network = dynamic_cast<NetworkInterface*>(module);
		}
	}

	void initInputInterfaceCallback(ModuleInterface* moduleInterface) 
	{
#ifdef WITH_VRPN_SUPPORT
		if (moduleInterface->getName() == "ControllerManager") {
			ControllerManager* contInt = dynamic_cast<ControllerManager*>(moduleInterface);
			assert(contInt);
			contInt->registerInputDeviceFactory(new VrpnDeviceFactory);
		} 
#endif
#ifdef WITH_TRACKD_SUPPORT
		if (moduleInterface->getName() == "ControllerManager") {
			ControllerManager* contInt = dynamic_cast<ControllerManager*>(moduleInterface);
			assert(contInt);
			contInt->registerInputDeviceFactory(new TrackdDeviceFactory);
		} 
#endif
	} 
};

int main(int argc, char** argv) {
	PictureCave* app = new PictureCave();

	if (!app->start(argc, argv)) {
		printd(ERROR, "Error occured during startup!\n");
		delete app;
		return -1;
	} 

	delete app;
	return 0;
} 
